/**
* Lightweight HTML5 WebSocket via Flash by Alexey Y. Bondar. February 1, 2010
* http://github.com/y8/websocket-flash-js
* @author Alexey Y. Bondar (y8 at ya dot ru), 2010.
* @see LICENSE http://github.com/y8/websocket-flash-jss/raw/master/LICENSE
*/
package {
    import flash.display.Sprite;
    import flash.events.Event;
    import flash.external.ExternalInterface;

    import y8.events.WebSocketErrorEvent;
    import y8.events.WebSocketEvent;
    import y8.net.WebSocket;

    public function _log(msg)
    {
      ExternalInterface.call("console.log",msg);
      //ExternalInterface.call("alert",msg);
    };
    
    /**
     * WebSocket class
     *
     * @author                    y8
     * @version                   1.0
     * @playerversion             Flash 9
     */
    public class WebSocketFlash extends Sprite {
        /**
         * @private
         */
        private var connections:Object;
        
        public var jsDispatcherName:String = "window.WebSocket.prototype.__dispatchFlashEvent";

        public function WebSocketFlash():void {
            _log("0000");

            this.connections = new Object();
            this.connections.nextId=0;
            this.connections.push=function(obj)
            {
              var id=this.nextId++;
              
              this[id]=obj;
              
              return id;
            };
            
            _log("1111");

            if(ExternalInterface && ExternalInterface.available) {
                ExternalInterface.call(jsDispatcherName, "__flashInited");
            }
        }
        /**
         * Creates WebSocket instance and return their id.
         * @return  id:String WebSocket instance
         */
        public function create():void {
            var websocket:WebSocket = new WebSocket();
            var item=
            {
              socket:new WebSocket(),
              __gcOnOpen:function():void {
                onOpen(id);
              },
              __gcOnClose:function():void {
                onClose(id);
              },
              __gcOnMessage:function(event:WebSocketEvent):void {
                onMessage(id, event.data);
              },
              __gcOnError:function(event:WebSocketErrorEvent):void {
                onError(id, event.error);
              }
            };

            var id:Number = this.connections.push(websocket);

            websocket.addEventListener(Event.OPEN, item.__gcOnOpen);

            websocket.addEventListener(Event.CLOSE, item.__gcOnClose);

            websocket.addEventListener(WebSocketEvent.MESSAGE, item.__gcOnMessage);

            websocket.addEventListener(WebSocketErrorEvent.ERROR, item.__gcOnError);

            return id;
        }

        /**
         * Destorys WebSocket instance.
         *
         * @param id:String — Connection id
         */
        public function destroy(id:Number):void {
            if(this.connections[id]) {
                var c=this.connections[id];
                
                c.removeEventListener(Event.OPEN,c.__gcOnOpen);
                c.removeEventListener(Event.CLOSE,c.__gcOnClose);
                c.removeEventListener(WebSocketEvent.MESSAGE,c.__gcOnMessage);
                c.removeEventListener(WebSocketErrorEvent.ERROR,c.__gcOnError);
                
                c.destory(); // lol
                
                this.connections[id] = null;
            } else {
                //ExternalInterface.call(jsDispatcherName, "not_found", id);
            }
        }

        /**
         * Opens WebSocket connection to given uri.
         *
         * @param id:String — Connection id
         * @param uri:String — WebSocket URI pointing to server, only "ws" scheme supported.
         *
         * @event open:Event  — Dispatched after handshake complete.
         * @event close:Event — Dispatched when WebSocket connection closed.
         * @event error:WebSocketErrorEvent — Dispatched on WebSocket error
         * @event message:WebSocketEvent — Dispatched websocket dataframe recivied.
         */
        public function open(id:Number, uri:String):void {
            if(this.connections[id]) {
                this.connections[id].open(uri, root.loaderInfo.url);
            } else {
                //ExternalInterface.call(jsDispatcherName, "not_found", id);
            }
        }

        //Close connection
        public function close(id:Number):void {
            if(this.connections[id]) {
                this.connections[id].close();
            } else {
                //ExternalInterface.call(jsDispatcherName, "not_found", id);
            }
        }
        
        /**
         * @private
         */
        private function onOpen(id:Number):void {
            ExternalInterface.call(jsDispatcherName, "__flashWSOpened", id);
        };
        
        /**
         * @private
         */
        private function onClose(id:Number):void {
            ExternalInterface.call(jsDispatcherName, "__flashWSClosed", id);
        };
        
        /**
         * @private
         */
        private function onMessage(id:Number, data:String):void {
            ExternalInterface.call(jsDispatcherName, "__flashWSMessageReceived", id, data);
        };
        
        /**
         * @private
         */
        private function onError(id:Number, text:String):void {
            ExternalInterface.call(jsDispatcherName, "__flashWSError", id, text);
        };
    }
    
    var temp=new WebSocketFlash();
}